
#include "MovieWriter.h"


extern void Ding(char *msg, HRESULT hr);

#define DINGRET(x) if FAILED(hr) { Ding(x, hr); return(hr); }

#define TICKS_PER_SEC ((double)(1.0E+7))


MovieWriter::MovieWriter() {
	mProfile = NULL;
	mWriter = NULL;
	mSample = NULL;
	WMCreateProfileManager(&mManager);
}

MovieWriter::~MovieWriter() {
	SAFE_RELEASE(mProfile);
	SAFE_RELEASE(mWriter);
	SAFE_RELEASE(mSample);
}


HRESULT MovieWriter::WriterProfileMerge() {
	HRESULT hr=S_OK;

	hr = mWriter->SetProfile(mProfile);
	DINGRET("Could not associate profile with writer");

	hr = mWriter->SetOutputFilename(lclFilename);
	DINGRET("Didn't like the output file name");

	hr = mWriter->BeginWriting();
	DINGRET("Could not begin writing file");

	return(hr);
}


HRESULT MovieWriter::Open(const WCHAR *filename) {
	HRESULT	hr=S_OK;

	wcscpy(lclFilename, filename);

	if (mWriter == NULL) {
		hr = WMCreateWriter(NULL, &mWriter);
		DINGRET("Could not create Writer");
	}

	if (mProfile != NULL) hr=WriterProfileMerge();
	
	return(hr);
}

HRESULT MovieWriter::Close(void) {
	HRESULT	hr=S_OK;

	if (mWriter != NULL) {
		hr = mWriter->EndWriting();
		DINGRET("Could not end writing");
	}

	return(hr);
}


HRESULT MovieWriter::SetOutfile(CString &filename) {
	WCHAR	filenameW[1000];
	LPCSTR	filenameC = filename;

	mbstowcs(filenameW, filenameC, strlen(filenameC)+1);
	Close();
	return(Open(filenameW));
}


HRESULT MovieWriter::SaveFrame(INSSBuffer *savesample, double timesample) {
	HRESULT	hr = S_OK;
	QWORD	timeq;

	timeq = (QWORD)(timesample*1E7);
	if (mWriter != NULL) {
		hr = mWriter->WriteSample(0, timeq, 0, savesample);
		DINGRET("Could not write sample");
	}

	return(hr);
}

HRESULT MovieWriter::InitProfile(IWMProfile *inprof) {
	HRESULT hr = S_OK;
	IWMStreamConfig		*pStream = NULL;
	DWORD				dwStreams = 0, profileLen = 0;
	GUID				pguidStreamType;


	// Set local profile to input profile to start
	if (mProfile != NULL) SAFE_RELEASE(mProfile);
	mManager->SaveProfile(inprof, NULL, &profileLen);
	WCHAR *profileText = new WCHAR[profileLen];
	char *profileTextC = new char[profileLen+1];
	mManager->SaveProfile(inprof, profileText, &profileLen);
	wcstombs(profileTextC, profileText, profileLen);
	*(profileTextC+profileLen)='\0';
	FILE *f=fopen("lastprofile.xml", "w");
	fprintf(f, "%s", profileTextC);
	fclose(f);
	mManager->LoadProfileByData(profileText, &mProfile);
	delete[] profileText;
	delete[] profileTextC;

	// Eliminate all but the video stream
	hr = mProfile->GetStreamCount(&dwStreams);
	DINGRET("Could not get stream count");

	for (DWORD i=0; i<dwStreams; i++) {
		hr = mProfile->GetStream(i, &pStream);
		DINGRET("Could not get an individual stream");

		hr = pStream->GetStreamType(&pguidStreamType);
		DINGRET("Could not get an individual stream type");

		if (pguidStreamType != WMMEDIATYPE_Video) {
			hr = mProfile->RemoveStream(pStream);
			DINGRET("Could not remove non-video stream");
			i--;
			hr = mProfile->GetStreamCount(&dwStreams);
			DINGRET("Could not get stream count after eliminating one");
		}
		else {
			hr = pStream->GetStreamNumber(&VideoStreamNum);
			DINGRET("Could not get stream number");
		}
	}

	if (mWriter != NULL) WriterProfileMerge();

	return(hr);
}



